@@ -9,7 +9,7 @@ use pango_cairo_sys;
 use pango_sys;
 use pangocairo;
 use std::cell::RefCell;
-use std::rc::Rc;
+use std::rc::{Rc, Weak};
 
 use bbox::BoundingBox;
 use clip_path::{ClipPathUnits, NodeClipPath};
@@ -39,6 +39,63 @@ use tree::{RsvgTree, Tree};
 use unitinterval::UnitInterval;
 use viewbox::ViewBox;
 
+/// Holds values that are required to normalize `Length` values to a current viewport.
+///
+/// This struct is created by calling `DrawingCtx::push_view_box()` or
+/// `DrawingCtx::get_view_params()`.
+///
+/// This struct holds the size of the current viewport in the user's coordinate system.  A
+/// viewport pushed with `DrawingCtx::push_view_box()` will remain in place until the
+/// returned `ViewParams` is dropped; at that point, the `DrawingCtx` will resume using its
+/// previous viewport.
+pub struct ViewParams {
+    dpi_x: f64,
+    dpi_y: f64,
+    view_box_width: f64,
+    view_box_height: f64,
+    view_box_stack: Option<Weak<RefCell<Vec<ViewBox>>>>,
+}
+
+impl ViewParams {
+    #[cfg(test)]
+    pub fn new(dpi_x: f64, dpi_y: f64, view_box_width: f64, view_box_height: f64) -> ViewParams {
+        ViewParams {
+            dpi_x,
+            dpi_y,
+            view_box_width,
+            view_box_height,
+            view_box_stack: None,
+        }
+    }
+
+    pub fn dpi_x(&self) -> f64 {
+        self.dpi_x
+    }
+
+    pub fn dpi_y(&self) -> f64 {
+        self.dpi_y
+    }
+
+    pub fn view_box_width(&self) -> f64 {
+        self.view_box_width
+    }
+
+    pub fn view_box_height(&self) -> f64 {
+        self.view_box_height
+    }
+}
+
+impl Drop for ViewParams {
+    fn drop(&mut self) {
+        if let Some(ref weak_stack) = self.view_box_stack {
+            let stack = weak_stack
+                .upgrade()
+                .expect("A ViewParams was dropped after its DrawingCtx!?");
+            stack.borrow_mut().pop();
+        }
+    }
+}
+
 pub enum RsvgDrawingCtx {}
 
 pub struct DrawingCtx<'a> {
@@ -64,8 +121,7 @@ pub struct DrawingCtx<'a> {
 
     surfaces_stack: Vec<cairo::ImageSurface>,
 
-    vb: ViewBox,
-    vb_stack: Vec<ViewBox>,
+    view_box_stack: Rc<RefCell<Vec<ViewBox>>>,
 
     bbox: BoundingBox,
     bbox_stack: Vec<BoundingBox>,
@@ -89,7 +145,7 @@ impl<'a> DrawingCtx<'a> {
         dpi_y: f64,
         defs: &mut Defs,
         is_testing: bool,
-    ) -> DrawingCtx {
+    ) -> DrawingCtx<'_> {
         let mut affine = cr.get_matrix();
         let rect = cairo::Rectangle {
             x: 0.0,
@@ -110,6 +166,9 @@ impl<'a> DrawingCtx<'a> {
         affine.y0 -= rect.y;
         cr.set_matrix(affine);
 
+        let mut view_box_stack = Vec::new();
+        view_box_stack.push(ViewBox::new(0.0, 0.0, vb_width, vb_height));
+
         DrawingCtx {
             rect,
             dpi_x,
@@ -119,8 +178,7 @@ impl<'a> DrawingCtx<'a> {
             cr: cr.clone(),
             initial_cr: cr.clone(),
             surfaces_stack: Vec::new(),
-            vb: ViewBox::new(0.0, 0.0, vb_width, vb_height),
-            vb_stack: Vec::new(),
+            view_box_stack: Rc::new(RefCell::new(view_box_stack)),
             bbox: BoundingBox::new(&affine),
             bbox_stack: Vec::new(),
             drawsub_stack: Vec::new(),
@@ -171,21 +229,40 @@ impl<'a> DrawingCtx<'a> {
         }
     }
 
-    pub fn get_dpi(&self) -> (f64, f64) {
-        (self.dpi_x, self.dpi_y)
+    /// Gets the viewport that was last pushed with `push_view_box()`.
+    pub fn get_view_params(&self) -> ViewParams {
+        let view_box_stack = self.view_box_stack.borrow();
+        let last = view_box_stack.len() - 1;
+        let stack_top = &view_box_stack[last];
+
+        ViewParams {
+            dpi_x: self.dpi_x,
+            dpi_y: self.dpi_y,
+            view_box_width: stack_top.0.width,
+            view_box_height: stack_top.0.height,
+            view_box_stack: None,
+        }
     }
 
-    pub fn get_view_box_size(&self) -> (f64, f64) {
-        (self.vb.0.width, self.vb.0.height)
-    }
+    /// Pushes a viewport size for normalizing `Length` values.
+    ///
+    /// You should pass the returned `ViewParams` to all subsequent `Length.normalize()`
+    /// calls that correspond to this viewport.
+    ///
+    /// The viewport will stay in place, and will be the one returned by
+    /// `get_view_params()`, until the returned `ViewParams` is dropped.
+    pub fn push_view_box(&self, width: f64, height: f64) -> ViewParams {
+        self.view_box_stack
+            .borrow_mut()
+            .push(ViewBox::new(0.0, 0.0, width, height));
 
-    pub fn push_view_box(&mut self, width: f64, height: f64) {
-        self.vb_stack.push(self.vb);
-        self.vb = ViewBox::new(0.0, 0.0, width, height);
-    }
-
-    pub fn pop_view_box(&mut self) {
-        self.vb = self.vb_stack.pop().unwrap();
+        ViewParams {
+            dpi_x: self.dpi_x,
+            dpi_y: self.dpi_y,
+            view_box_width: width,
+            view_box_height: height,
+            view_box_stack: Some(Rc::downgrade(&self.view_box_stack)),
+        }
     }
 
     pub fn insert_bbox(&mut self, bbox: &BoundingBox) {
@@ -213,7 +290,8 @@ impl<'a> DrawingCtx<'a> {
         if let Some(node) = self.defs.borrow_mut().lookup(url) {
             if !self.acquired_nodes_contains(node) {
                 self.acquired_nodes.borrow_mut().push(node.clone());
-                return Some(AcquiredNode(self.acquired_nodes.clone(), node.clone()));
+                let acq = AcquiredNode(self.acquired_nodes.clone(), node.clone());
+                return Some(acq);
             }
         }
 
@@ -262,7 +340,7 @@ impl<'a> DrawingCtx<'a> {
         node: &RsvgNode,
         values: &ComputedValues,
         clipping: bool,
-        draw_fn: &mut FnMut(&mut DrawingCtx) -> Result<(), RenderingError>,
+        draw_fn: &mut FnMut(&mut DrawingCtx<'_>) -> Result<(), RenderingError>,
     ) -> Result<(), RenderingError> {
         if clipping {
             draw_fn(self)
@@ -319,14 +397,11 @@ impl<'a> DrawingCtx<'a> {
                 && enable_background == EnableBackground::Accumulate);
 
             if needs_temporary_surface {
-                // FIXME: in the following, we unwrap() the result of
-                // ImageSurface::create().  We have to decide how to handle
-                // out-of-memory here.
                 let surface = cairo::ImageSurface::create(
                     cairo::Format::ARgb32,
                     self.rect.width as i32,
                     self.rect.height as i32,
-                ).unwrap();
+                )?;
 
                 if filter.is_some() {
                     self.surfaces_stack.push(surface.clone());
@@ -347,43 +422,11 @@ impl<'a> DrawingCtx<'a> {
             if needs_temporary_surface {
                 let child_surface = cairo::ImageSurface::from(self.cr.get_target()).unwrap();
 
-                let filter_result_surface = filter
-                    .and_then(|_| {
-                        // About the following unwrap(), see the FIXME above.  We should be pushing
-                        // only surfaces that are not in an error state, but currently we don't
-                        // actually ensure that.
-                        let output = self.surfaces_stack.pop().unwrap();
-
-                        match self.get_acquired_node_of_type(filter, NodeType::Filter) {
-                            Some(acquired) => {
-                                let filter_node = acquired.get();
-
-                                if !filter_node.is_in_error() {
-                                    // FIXME: deal with out of memory here
-                                    Some(filters::render(&filter_node, node, &output, self))
-                                } else {
-                                    rsvg_log!(
-                                        "(ignoring filter element {} because it is in error)",
-                                        filter_node.get_human_readable_name()
-                                    );
-                                    None
-                                }
-                            }
-                            None => {
-                                // Non-existing filters must act as null filters (that is, an
-                                // empty surface is returned).
-                                // TODO: handle the error properly and not via expect().
-                                Some(
-                                    cairo::ImageSurface::create(
-                                        cairo::Format::ARgb32,
-                                        child_surface.get_width(),
-                                        child_surface.get_height(),
-                                    ).expect("couldn't create an empty surface"),
-                                )
-                            }
-                        }
-                    }).or(Some(child_surface))
-                    .unwrap();
+                let filter_result_surface = if let Some(filter_uri) = filter {
+                    self.run_filter(filter_uri, values, &child_surface)?
+                } else {
+                    child_surface
+                };
 
                 self.cr = self.cr_stack.pop().unwrap();
 
@@ -435,6 +478,38 @@ impl<'a> DrawingCtx<'a> {
         }
     }
 
+    fn run_filter(
+        &mut self,
+        filter_uri: &str,
+        values: &ComputedValues,
+        child_surface: &cairo::ImageSurface,
+    ) -> Result<cairo::ImageSurface, RenderingError> {
+        let output = self.surfaces_stack.pop().unwrap();
+
+        match self.get_acquired_node_of_type(Some(filter_uri), NodeType::Filter) {
+            Some(acquired) => {
+                let filter_node = acquired.get();
+
+                if !filter_node.is_in_error() {
+                    // FIXME: deal with out of memory here
+                    Ok(filters::render(&filter_node, values, &output, self))
+                } else {
+                    Ok(child_surface.clone())
+                }
+            }
+
+            None => {
+                // Non-existing filters must act as null filters (that is, an
+                // empty surface is returned).
+                Ok(cairo::ImageSurface::create(
+                    cairo::Format::ARgb32,
+                    child_surface.get_width(),
+                    child_surface.get_height(),
+                )?)
+            }
+        }
+    }
+
     pub fn get_pango_context(&self) -> pango::Context {
         let font_map = pangocairo::FontMap::get_default().unwrap();
         let context = font_map.create_context().unwrap();
@@ -559,19 +634,23 @@ impl<'a> DrawingCtx<'a> {
     }
 
     fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
-        cr.set_line_width(values.stroke_width.0.normalize(values, self));
+        let params = self.get_view_params();
+
+        cr.set_line_width(values.stroke_width.0.normalize(values, &params));
         cr.set_miter_limit(values.stroke_miterlimit.0);
         cr.set_line_cap(cairo::LineCap::from(values.stroke_line_cap));
         cr.set_line_join(cairo::LineJoin::from(values.stroke_line_join));
 
         if let StrokeDasharray(Dasharray::Array(ref dashes)) = values.stroke_dasharray {
-            let normalized_dashes: Vec<f64> =
-                dashes.iter().map(|l| l.normalize(values, self)).collect();
+            let normalized_dashes: Vec<f64> = dashes
+                .iter()
+                .map(|l| l.normalize(values, &params))
+                .collect();
 
             let total_length = normalized_dashes.iter().fold(0.0, |acc, &len| acc + len);
 
             if total_length > 0.0 {
-                let offset = values.stroke_dashoffset.0.normalize(values, self);
+                let offset = values.stroke_dashoffset.0.normalize(values, &params);
                 cr.set_dash(&normalized_dashes, offset);
             } else {
                 cr.set_dash(&[], 0.0);
@@ -667,7 +746,7 @@ impl<'a> DrawingCtx<'a> {
     pub fn draw_node_on_surface(
         &mut self,
         node: &RsvgNode,
-        cascaded: &CascadedValues,
+        cascaded: &CascadedValues<'_>,
         surface: &cairo::ImageSurface,
         width: f64,
         height: f64,
@@ -698,7 +777,7 @@ impl<'a> DrawingCtx<'a> {
 
     pub fn draw_node_from_stack(
         &mut self,
-        cascaded: &CascadedValues,
+        cascaded: &CascadedValues<'_>,
         node: &RsvgNode,
         clipping: bool,
     ) -> Result<(), RenderingError> {
@@ -960,7 +1039,7 @@ pub extern "C" fn rsvg_drawing_ctx_draw_node_from_stack(
     raw_tree: *const RsvgTree,
 ) -> glib_sys::gboolean {
     assert!(!raw_draw_ctx.is_null());
-    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx) };
+    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx<'_>) };
 
     assert!(!raw_tree.is_null());
     let tree = unsafe { &*(raw_tree as *const Tree) };
@@ -984,7 +1063,7 @@ pub extern "C" fn rsvg_drawing_ctx_add_node_and_ancestors_to_stack(
     raw_node: *const RsvgNode,
 ) {
     assert!(!raw_draw_ctx.is_null());
-    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx) };
+    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx<'_>) };
 
     assert!(!raw_node.is_null());
     let node = unsafe { &*raw_node };
@@ -998,7 +1077,7 @@ pub extern "C" fn rsvg_drawing_ctx_get_ink_rect(
     ink_rect: *mut cairo_sys::cairo_rectangle_t,
 ) -> glib_sys::gboolean {
     assert!(!raw_draw_ctx.is_null());
-    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx) };
+    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx<'_>) };
 
     assert!(!ink_rect.is_null());
 
@@ -1032,6 +1111,27 @@ impl AcquiredNode {
     }
 }
 
+/// Keeps a stack of nodes and can check if a certain node is contained in the stack
+///
+/// Sometimes parts of the code cannot plainly use the implicit stack of acquired
+/// nodes as maintained by DrawingCtx::get_acquired_node(), and they must keep their
+/// own stack of nodes to test for reference cycles.  NodeStack can be used to do that.
+pub struct NodeStack(Vec<RsvgNode>);
+
+impl NodeStack {
+    pub fn new() -> NodeStack {
+        NodeStack(Vec::new())
+    }
+
+    pub fn push(&mut self, node: &RsvgNode) {
+        self.0.push(node.clone());
+    }
+
+    pub fn contains(&self, node: &RsvgNode) -> bool {
+        self.0.iter().find(|n| Rc::ptr_eq(n, node)).is_some()
+    }
+}
+
 #[no_mangle]
 pub extern "C" fn rsvg_drawing_ctx_new(
     cr: *mut cairo_sys::cairo_t,
@@ -1063,7 +1163,7 @@ pub extern "C" fn rsvg_drawing_ctx_new(
 #[no_mangle]
 pub extern "C" fn rsvg_drawing_ctx_free(raw_draw_ctx: *mut RsvgDrawingCtx) {
     assert!(!raw_draw_ctx.is_null());
-    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx) };
+    let draw_ctx = unsafe { &mut *(raw_draw_ctx as *mut DrawingCtx<'_>) };
 
     unsafe {
         Box::from_raw(draw_ctx);
