@@ -25,8 +25,6 @@ extern crate lazy_static;
 
 extern crate downcast_rs;
 
-pub use attributes::rsvg_attribute_from_name;
-
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
 pub use defs::{rsvg_defs_free, rsvg_defs_lookup, rsvg_defs_new};
@@ -71,7 +69,7 @@ pub use property_bag::{
     rsvg_property_bag_new,
 };
 
-pub use state::{rsvg_state_free, rsvg_state_new, rsvg_state_parse_style_pair};
+pub use state::rsvg_state_parse_style_pair;
 
 pub use structure::rsvg_node_svg_get_size;
 
