@@ -3,7 +3,7 @@ use downcast_rs::*;
 use glib;
 use glib::translate::*;
 use glib_sys;
-
+use libc;
 use std::cell::{Cell, Ref, RefCell};
 use std::ptr;
 use std::rc::{Rc, Weak};
@@ -15,16 +15,15 @@ use error::*;
 use handle::RsvgHandle;
 use parsers::Parse;
 use property_bag::PropertyBag;
-use state::{
-    self,
-    rsvg_state_new,
-    ComputedValues,
-    Overflow,
-    RsvgState,
-    SpecifiedValue,
-    SpecifiedValues,
-    State,
-};
+use state::{ComputedValues, Overflow, RsvgState, SpecifiedValue, State};
+
+extern "C" {
+    fn rsvg_lookup_apply_css_style(
+        handle: *const RsvgHandle,
+        target: *const libc::c_char,
+        state: *mut RsvgState,
+    ) -> glib_sys::gboolean;
+}
 
 // A *const RsvgNode is just a pointer for the C code's benefit: it
 // points to an  Rc<Node>, which is our refcounted Rust representation
@@ -71,7 +70,7 @@ impl<'a> CascadedValues<'a> {
     /// This is to be used only in the toplevel drawing function, or in elements like `<marker>`
     /// that don't propagate their parent's cascade to their children.  All others should use
     /// `new()` to derive the cascade from an existing one.
-    fn new_from_node(node: &Node) -> CascadedValues {
+    fn new_from_node(node: &Node) -> CascadedValues<'_> {
         CascadedValues {
             inner: CascadedInner::FromNode(node.values.borrow()),
         }
@@ -83,8 +82,9 @@ impl<'a> CascadedValues<'a> {
     /// This is for the `<use>` element, which draws the element which it references with the
     /// `<use>`'s own cascade, not wih the element's original cascade.
     pub fn new_from_values(node: &'a Node, values: &ComputedValues) -> CascadedValues<'a> {
+        let state = node.state.borrow();
         let mut v = values.clone();
-        node.get_specified_values().to_computed_values(&mut v);
+        state.get_specified_values().to_computed_values(&mut v);
 
         CascadedValues {
             inner: CascadedInner::FromValues(v),
@@ -112,7 +112,7 @@ pub trait NodeTrait: Downcast {
         &self,
         node: &RsvgNode,
         handle: *const RsvgHandle,
-        pbag: &PropertyBag,
+        pbag: &PropertyBag<'_>,
     ) -> NodeResult;
 
     /// Sets any special-cased properties that the node may have, that are different
@@ -126,8 +126,8 @@ pub trait NodeTrait: Downcast {
     fn draw(
         &self,
         _node: &RsvgNode,
-        _cascaded: &CascadedValues,
-        _draw_ctx: &mut DrawingCtx,
+        _cascaded: &CascadedValues<'_>,
+        _draw_ctx: &mut DrawingCtx<'_>,
         _clipping: bool,
     ) -> Result<(), RenderingError> {
         // by default nodes don't draw themselves
@@ -162,13 +162,14 @@ pub type NodeResult = Result<(), NodeError>;
 pub struct Node {
     node_type: NodeType,
     parent: Option<Weak<Node>>, // optional; weak ref to parent
+    element_name: String,       // we may want to intern these someday
     id: Option<String>,         // id attribute from XML element
     class: Option<String>,      // class attribute from XML element
     first_child: RefCell<Option<Rc<Node>>>,
     last_child: RefCell<Option<Weak<Node>>>,
     next_sib: RefCell<Option<Rc<Node>>>, // next sibling; strong ref
     prev_sib: RefCell<Option<Weak<Node>>>, // previous sibling; weak ref
-    state: *mut RsvgState,
+    state: RefCell<State>,
     result: RefCell<NodeResult>,
     transform: Cell<Matrix>,
     values: RefCell<ComputedValues>,
@@ -243,21 +244,22 @@ impl Node {
     pub fn new(
         node_type: NodeType,
         parent: Option<Weak<Node>>,
+        element_name: &str,
         id: Option<&str>,
         class: Option<&str>,
-        state: *mut RsvgState,
         node_impl: Box<NodeTrait>,
     ) -> Node {
         Node {
             node_type,
             parent,
+            element_name: element_name.to_string(),
             id: id.map(str::to_string),
             class: class.map(str::to_string),
             first_child: RefCell::new(None),
             last_child: RefCell::new(None),
             next_sib: RefCell::new(None),
             prev_sib: RefCell::new(None),
-            state,
+            state: RefCell::new(State::new()),
             transform: Cell::new(Matrix::identity()),
             result: RefCell::new(Ok(())),
             values: RefCell::new(ComputedValues::default()),
@@ -290,21 +292,14 @@ impl Node {
         self.transform.get()
     }
 
-    pub fn get_state_mut(&self) -> &mut State {
-        state::from_c_mut(self.state)
-    }
-
-    pub fn get_specified_values(&self) -> &SpecifiedValues {
-        state::from_c(self.state).get_specified_values()
-    }
-
-    pub fn get_cascaded_values(&self) -> CascadedValues {
+    pub fn get_cascaded_values(&self) -> CascadedValues<'_> {
         CascadedValues::new_from_node(self)
     }
 
     pub fn cascade(&self, values: &ComputedValues) {
+        let state = self.state.borrow();
         let mut values = values.clone();
-        self.get_specified_values().to_computed_values(&mut values);
+        state.get_specified_values().to_computed_values(&mut values);
         *self.values.borrow_mut() = values.clone();
 
         for child in self.children() {
@@ -351,7 +346,7 @@ impl Node {
         self.first_child.replace(Some(child.clone()));
     }
 
-    pub fn set_atts(&self, node: &RsvgNode, handle: *const RsvgHandle, pbag: &PropertyBag) {
+    pub fn set_atts(&self, node: &RsvgNode, handle: *const RsvgHandle, pbag: &PropertyBag<'_>) {
         for (_key, attr, value) in pbag.iter() {
             match attr {
                 Attribute::Transform => match Matrix::parse_str(value, ()) {
@@ -383,7 +378,10 @@ impl Node {
         }
     }
 
-    fn parse_conditional_processing_attributes(&self, pbag: &PropertyBag) -> Result<(), NodeError> {
+    fn parse_conditional_processing_attributes(
+        &self,
+        pbag: &PropertyBag<'_>,
+    ) -> Result<(), NodeError> {
         let mut cond = self.cond.get();
 
         for (_key, attr, value) in pbag.iter() {
@@ -419,16 +417,138 @@ impl Node {
         Ok(())
     }
 
+    /// Hands the pbag to the node's state, to apply the presentation attributes
+    fn set_presentation_attributes(&self, pbag: &PropertyBag<'_>) {
+        let mut state = self.state.borrow_mut();
+        match state.parse_presentation_attributes(pbag) {
+            Ok(_) => (),
+            Err(e) => {
+                // FIXME: we'll ignore errors here for now.
+                //
+                // If we set the node to be in error, we expose buggy handling of the
+                // enable-background property; we are not parsing it correctly. This
+                // causes tests/fixtures/reftests/bugs/587721-text-transform.svg to fail
+                // because it has enable-background="new 0 0 1179.75118 687.74173" in the
+                // toplevel svg element.
+                //
+                //   self.set_error(e);
+                //   return;
+
+                rsvg_log!("(attribute error: {})", e);
+            }
+        }
+    }
+
+    /// Implements a very limited CSS selection engine
+    fn set_css_styles(&self, handle: *const RsvgHandle) {
+        // Try to properly support all of the following, including inheritance:
+        // *
+        // #id
+        // tag
+        // tag#id
+        // tag.class
+        // tag.class#id
+        //
+        // This is basically a semi-compliant CSS2 selection engine
+
+        unsafe {
+            let state_ptr = self.state.as_ptr() as *mut RsvgState;
+
+            // *
+            rsvg_lookup_apply_css_style(handle, "*".to_glib_none().0, state_ptr);
+
+            // tag
+            rsvg_lookup_apply_css_style(handle, self.element_name.to_glib_none().0, state_ptr);
+
+            if let Some(klazz) = self.get_class() {
+                for cls in klazz.split_whitespace() {
+                    let mut found = false;
+
+                    if !cls.is_empty() {
+                        // tag.class#id
+                        if let Some(id) = self.get_id() {
+                            let target = format!("{}.{}#{}", self.element_name, cls, id);
+                            found = found || from_glib(rsvg_lookup_apply_css_style(
+                                handle,
+                                target.to_glib_none().0,
+                                state_ptr,
+                            ));
+                        }
+
+                        // .class#id
+                        if let Some(id) = self.get_id() {
+                            let target = format!(".{}#{}", cls, id);
+                            found = found || from_glib(rsvg_lookup_apply_css_style(
+                                handle,
+                                target.to_glib_none().0,
+                                state_ptr,
+                            ));
+                        }
+
+                        // tag.class
+                        let target = format!("{}.{}", self.element_name, cls);
+                        found = found || from_glib(rsvg_lookup_apply_css_style(
+                            handle,
+                            target.to_glib_none().0,
+                            state_ptr,
+                        ));
+
+                        if !found {
+                            // didn't find anything more specific, just apply the class style
+                            let target = format!(".{}", cls);
+                            rsvg_lookup_apply_css_style(handle, target.to_glib_none().0, state_ptr);
+                        }
+                    }
+                }
+            }
+
+            if let Some(id) = self.get_id() {
+                // id
+                let target = format!("#{}", id);
+                rsvg_lookup_apply_css_style(handle, target.to_glib_none().0, state_ptr);
+
+                // tag#id
+                let target = format!("{}#{}", self.element_name, id);
+                rsvg_lookup_apply_css_style(handle, target.to_glib_none().0, state_ptr);
+            }
+        }
+    }
+
+    /// Looks for the "style" attribute in the pbag, and applies CSS styles from it
+    fn set_style_attribute(&self, pbag: &PropertyBag<'_>) {
+        for (_key, attr, value) in pbag.iter() {
+            match attr {
+                Attribute::Style => {
+                    let mut state = self.state.borrow_mut();
+                    if let Err(e) = state.parse_style_declarations(value) {
+                        self.set_error(e);
+                        break;
+                    }
+                }
+
+                _ => (),
+            }
+        }
+    }
+
+    // Sets the node's state from the style-related attributes in the pbag.  Also applies
+    // CSS rules in our limited way based on the node's tag/class/id.
+    pub fn set_style(&self, handle: *const RsvgHandle, pbag: &PropertyBag<'_>) {
+        self.set_presentation_attributes(pbag);
+        self.set_css_styles(handle);
+        self.set_style_attribute(pbag);
+    }
+
     pub fn set_overridden_properties(&self) {
-        let mut state = self.get_state_mut();
+        let mut state = self.state.borrow_mut();
         self.node_impl.set_overridden_properties(&mut state);
     }
 
     pub fn draw(
         &self,
         node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         clipping: bool,
     ) -> Result<(), RenderingError> {
         if !self.is_in_error() {
@@ -484,8 +604,8 @@ impl Node {
 
     pub fn draw_children(
         &self,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         clipping: bool,
     ) -> Result<(), RenderingError> {
         for child in self.children() {
@@ -512,8 +632,13 @@ impl Node {
         self.first_child.borrow().is_some()
     }
 
+    pub fn is_overflow(&self) -> bool {
+        let state = self.state.borrow();
+        state.get_specified_values().is_overflow()
+    }
+
     pub fn set_overflow_hidden(&self) {
-        let state = self.get_state_mut();
+        let mut state = self.state.borrow_mut();
         state.values.overflow = SpecifiedValue::Specified(Overflow::Hidden);
     }
 
@@ -542,22 +667,6 @@ impl Node {
     }
 }
 
-// Sigh, rsvg_state_free() is only available if we are being linked into
-// librsvg.so.  In testing mode, we run standalone, so we omit this.
-// Fortunately, in testing mode we don't create "real" nodes with
-// states; we only create stub nodes with ptr::null() for state.
-#[cfg(not(test))]
-impl Drop for Node {
-    fn drop(&mut self) {
-        extern "C" {
-            fn rsvg_state_free(state: *mut RsvgState);
-        }
-        unsafe {
-            rsvg_state_free(self.state);
-        }
-    }
-}
-
 pub fn node_ptr_to_weak(raw_parent: *const RsvgNode) -> Option<Weak<Node>> {
     if raw_parent.is_null() {
         None
@@ -570,6 +679,7 @@ pub fn node_ptr_to_weak(raw_parent: *const RsvgNode) -> Option<Weak<Node>> {
 pub fn boxed_node_new(
     node_type: NodeType,
     raw_parent: *const RsvgNode,
+    element_name: &str,
     id: Option<&str>,
     class: Option<&str>,
     node_impl: Box<NodeTrait>,
@@ -577,9 +687,9 @@ pub fn boxed_node_new(
     box_node(Rc::new(Node::new(
         node_type,
         node_ptr_to_weak(raw_parent),
+        element_name,
         id,
         class,
-        rsvg_state_new(),
         node_impl,
     )))
 }
@@ -743,13 +853,13 @@ pub extern "C" fn rsvg_node_children_iter_next(
 mod tests {
     use super::*;
     use handle::RsvgHandle;
+    use std::mem;
     use std::rc::Rc;
-    use std::{mem, ptr};
 
     struct TestNodeImpl {}
 
     impl NodeTrait for TestNodeImpl {
-        fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag) -> NodeResult {
+        fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
             Ok(())
         }
     }
@@ -759,9 +869,9 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
@@ -785,9 +895,9 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
@@ -808,9 +918,9 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
@@ -822,18 +932,18 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
         let child = Rc::new(Node::new(
             NodeType::Path,
             Some(Rc::downgrade(&node)),
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
@@ -848,27 +958,27 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
         let child = Rc::new(Node::new(
             NodeType::Path,
             Some(Rc::downgrade(&node)),
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
         let second_child = Rc::new(Node::new(
             NodeType::Path,
             Some(Rc::downgrade(&node)),
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
@@ -896,27 +1006,27 @@ mod tests {
         let node = Rc::new(Node::new(
             NodeType::Path,
             None,
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
         let child = Rc::new(Node::new(
             NodeType::Path,
             Some(Rc::downgrade(&node)),
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
         let second_child = Rc::new(Node::new(
             NodeType::Path,
             Some(Rc::downgrade(&node)),
+            "path",
             None,
             None,
-            ptr::null_mut(),
             Box::new(TestNodeImpl {}),
         ));
 
