@@ -7,7 +7,7 @@ use std::f64::consts::*;
 use std::str::{self, FromStr};
 
 use attributes::Attribute;
-use error::{AttributeError, NodeError};
+use error::{NodeError, ValueErrorKind};
 use util::utf8_cstr;
 
 #[derive(Debug, Clone, PartialEq)]
@@ -24,7 +24,7 @@ impl ParseError {
 }
 
 impl<'a> From<BasicParseError<'a>> for ParseError {
-    fn from(_: BasicParseError) -> ParseError {
+    fn from(_: BasicParseError<'_>) -> ParseError {
         ParseError::new("parse error")
     }
 }
@@ -33,7 +33,7 @@ pub trait Parse: Sized {
     type Data;
     type Err;
 
-    fn parse(parser: &mut Parser, data: Self::Data) -> Result<Self, Self::Err>;
+    fn parse(parser: &mut Parser<'_, '_>, data: Self::Data) -> Result<Self, Self::Err>;
 
     fn parse_str(s: &str, data: Self::Data) -> Result<Self, Self::Err> {
         let mut input = ParserInput::new(s);
@@ -46,26 +46,46 @@ pub trait Parse: Sized {
     }
 }
 
+/// Extend `cssparser::Parser` with a `expect_finite_number` method,
+/// to avoid infinities.
+pub trait CssParserExt {
+    fn expect_finite_number(&mut self) -> Result<f32, ValueErrorKind>;
+}
+
+impl<'i, 't> CssParserExt for Parser<'i, 't> {
+    fn expect_finite_number(&mut self) -> Result<f32, ValueErrorKind> {
+        finite_f32(self.expect_number()?)
+    }
+}
+
+pub fn finite_f32(n: f32) -> Result<f32, ValueErrorKind> {
+    if n.is_finite() {
+        Ok(n)
+    } else {
+        Err(ValueErrorKind::Value("expected finite number".to_string()))
+    }
+}
+
 impl Parse for f64 {
     type Data = ();
-    type Err = AttributeError;
+    type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser, _: ()) -> Result<f64, AttributeError> {
-        Ok(f64::from(parser.expect_number().map_err(|_| {
-            AttributeError::Parse(ParseError::new("expected number"))
+    fn parse(parser: &mut Parser<'_, '_>, _: ()) -> Result<f64, ValueErrorKind> {
+        Ok(f64::from(parser.expect_finite_number().map_err(|_| {
+            ValueErrorKind::Parse(ParseError::new("expected number"))
         })?))
     }
 }
 
 impl Parse for String {
     type Data = ();
-    type Err = AttributeError;
+    type Err = ValueErrorKind;
 
-    fn parse(parser: &mut Parser, _: ()) -> Result<String, AttributeError> {
+    fn parse(parser: &mut Parser<'_, '_>, _: ()) -> Result<String, ValueErrorKind> {
         Ok(String::from(
             parser
                 .expect_string()
-                .map_err(|_| AttributeError::Parse(ParseError::new("expected number")))?
+                .map_err(|_| ValueErrorKind::Parse(ParseError::new("expected string")))?
                 .as_ref(),
         ))
     }
@@ -80,7 +100,7 @@ impl Parse for String {
 /// `LengthDir::Horizontal` for `data`, for example.
 pub fn parse<T>(key: &str, value: &str, data: <T as Parse>::Data) -> Result<T, NodeError>
 where
-    T: Parse<Err = AttributeError>,
+    T: Parse<Err = ValueErrorKind>,
 {
     let mut input = ParserInput::new(value);
     let mut parser = Parser::new(&mut input);
@@ -103,8 +123,8 @@ pub fn parse_and_validate<T, F>(
     validate: F,
 ) -> Result<T, NodeError>
 where
-    T: Parse<Err = AttributeError>,
-    F: FnOnce(T) -> Result<T, AttributeError>,
+    T: Parse<Err = ValueErrorKind>,
+    F: FnOnce(T) -> Result<T, ValueErrorKind>,
 {
     let mut input = ParserInput::new(value);
     let mut parser = Parser::new(&mut input);
@@ -121,7 +141,7 @@ where
 //
 // Returns an f64 angle in degrees
 
-pub fn angle_degrees(parser: &mut Parser) -> Result<f64, ParseError> {
+pub fn angle_degrees(parser: &mut Parser<'_, '_>) -> Result<f64, ParseError> {
     let angle = {
         let token = parser
             .next()
@@ -154,29 +174,29 @@ pub fn angle_degrees(parser: &mut Parser) -> Result<f64, ParseError> {
     Ok(angle)
 }
 
-pub fn optional_comma(parser: &mut Parser) {
+pub fn optional_comma(parser: &mut Parser<'_, '_>) {
     let _ = parser.try(|p| p.expect_comma());
 }
 
 // number
 //
 // https://www.w3.org/TR/SVG11/types.html#DataTypeNumber
-pub fn number(s: &str) -> Result<f64, ParseError> {
+pub fn number(s: &str) -> Result<f64, ValueErrorKind> {
     let mut input = ParserInput::new(s);
     let mut parser = Parser::new(&mut input);
 
-    Ok(f64::from(parser.expect_number()?))
+    Ok(f64::from(parser.expect_finite_number()?))
 }
 
 // number-optional-number
 //
 // https://www.w3.org/TR/SVG/types.html#DataTypeNumberOptionalNumber
 
-pub fn number_optional_number(s: &str) -> Result<(f64, f64), ParseError> {
+pub fn number_optional_number(s: &str) -> Result<(f64, f64), ValueErrorKind> {
     let mut input = ParserInput::new(s);
     let mut parser = Parser::new(&mut input);
 
-    let x = f64::from(parser.expect_number()?);
+    let x = f64::from(parser.expect_finite_number()?);
 
     if !parser.is_exhausted() {
         let state = parser.state();
@@ -186,7 +206,7 @@ pub fn number_optional_number(s: &str) -> Result<(f64, f64), ParseError> {
             _ => parser.reset(&state),
         };
 
-        let y = f64::from(parser.expect_number()?);
+        let y = f64::from(parser.expect_finite_number()?);
 
         parser.expect_exhausted()?;
 
@@ -199,7 +219,7 @@ pub fn number_optional_number(s: &str) -> Result<(f64, f64), ParseError> {
 // integer
 //
 // https://www.w3.org/TR/SVG11/types.html#DataTypeInteger
-pub fn integer(s: &str) -> Result<i32, ParseError> {
+pub fn integer(s: &str) -> Result<i32, ValueErrorKind> {
     let mut input = ParserInput::new(s);
     let mut parser = Parser::new(&mut input);
 
@@ -209,7 +229,7 @@ pub fn integer(s: &str) -> Result<i32, ParseError> {
 // integer-optional-integer
 //
 // Like number-optional-number but with integers.
-pub fn integer_optional_integer(s: &str) -> Result<(i32, i32), ParseError> {
+pub fn integer_optional_integer(s: &str) -> Result<(i32, i32), ValueErrorKind> {
     let mut input = ParserInput::new(s);
     let mut parser = Parser::new(&mut input);
 
@@ -267,18 +287,18 @@ pub extern "C" fn rsvg_css_parse_number_optional_number(
 // Parse a list-of-points as for polyline and polygon elements
 // https://www.w3.org/TR/SVG/shapes.html#PointsBNF
 
-pub fn list_of_points(string: &str) -> Result<Vec<(f64, f64)>, ParseError> {
+pub fn list_of_points(string: &str) -> Result<Vec<(f64, f64)>, ValueErrorKind> {
     let mut input = ParserInput::new(string);
     let mut parser = Parser::new(&mut input);
 
     let mut v = Vec::new();
 
     loop {
-        let x = f64::from(parser.expect_number()?);
+        let x = f64::from(parser.expect_finite_number()?);
 
         optional_comma(&mut parser);
 
-        let y = f64::from(parser.expect_number()?);
+        let y = f64::from(parser.expect_finite_number()?);
 
         v.push((x, y));
 
@@ -309,7 +329,10 @@ pub enum NumberListError {
     Parse(ParseError),
 }
 
-pub fn number_list(parser: &mut Parser, length: ListLength) -> Result<Vec<f64>, NumberListError> {
+pub fn number_list(
+    parser: &mut Parser<'_, '_>,
+    length: ListLength,
+) -> Result<Vec<f64>, NumberListError> {
     let n;
 
     match length {
@@ -333,7 +356,7 @@ pub fn number_list(parser: &mut Parser, length: ListLength) -> Result<Vec<f64>,
             optional_comma(parser);
         }
 
-        v.push(f64::from(parser.expect_number().map_err(|_| {
+        v.push(f64::from(parser.expect_finite_number().map_err(|_| {
             NumberListError::Parse(ParseError::new("expected number"))
         })?));
 
