@@ -67,7 +67,7 @@ impl NodeChars {
         &self,
         _node: &RsvgNode,
         values: &ComputedValues,
-        draw_ctx: &DrawingCtx,
+        draw_ctx: &DrawingCtx<'_>,
         length: &mut f64,
     ) {
         let s = self.string.borrow();
@@ -81,7 +81,7 @@ impl NodeChars {
         &self,
         _node: &RsvgNode,
         values: &ComputedValues,
-        draw_ctx: &mut DrawingCtx,
+        draw_ctx: &mut DrawingCtx<'_>,
         x: &mut f64,
         y: &mut f64,
         clipping: bool,
@@ -91,7 +91,10 @@ impl NodeChars {
         let (width, _) = layout.get_size();
 
         let baseline = f64::from(layout.get_baseline()) / f64::from(pango::SCALE);
-        let offset = baseline + values.baseline_shift.0.normalize(values, draw_ctx);
+        let offset = baseline + values
+            .baseline_shift
+            .0
+            .normalize(values, &draw_ctx.get_view_params());
 
         if values.text_gravity_is_vertical() {
             draw_ctx.draw_pango_layout(&layout, values, *x + offset, *y, clipping)?;
@@ -106,7 +109,7 @@ impl NodeChars {
 }
 
 impl NodeTrait for NodeChars {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, _: &PropertyBag<'_>) -> NodeResult {
         Ok(())
     }
 }
@@ -130,7 +133,7 @@ impl NodeText {
 }
 
 impl NodeTrait for NodeText {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
                 Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
@@ -151,16 +154,18 @@ impl NodeTrait for NodeText {
     fn draw(
         &self,
         node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         clipping: bool,
     ) -> Result<(), RenderingError> {
         let values = cascaded.get();
 
-        let mut x = self.x.get().normalize(values, draw_ctx);
-        let mut y = self.y.get().normalize(values, draw_ctx);
-        let mut dx = self.dx.get().normalize(values, draw_ctx);
-        let mut dy = self.dy.get().normalize(values, draw_ctx);
+        let params = draw_ctx.get_view_params();
+
+        let mut x = self.x.get().normalize(values, &params);
+        let mut y = self.y.get().normalize(values, &params);
+        let mut dx = self.dx.get().normalize(values, &params);
+        let mut dy = self.dy.get().normalize(values, &params);
 
         let anchor = values.text_anchor;
 
@@ -203,8 +208,8 @@ impl NodeTRef {
     fn measure(
         &self,
         _node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         length: &mut f64,
     ) -> bool {
         let l = self.link.borrow();
@@ -226,8 +231,8 @@ impl NodeTRef {
     fn render(
         &self,
         _node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         x: &mut f64,
         y: &mut f64,
         clipping: bool,
@@ -248,7 +253,7 @@ impl NodeTRef {
 }
 
 impl NodeTrait for NodeTRef {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
                 Attribute::XlinkHref => *self.link.borrow_mut() = Some(value.to_owned()),
@@ -280,8 +285,8 @@ impl NodeTSpan {
     fn measure(
         &self,
         node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         length: &mut f64,
         usetextonly: bool,
     ) -> bool {
@@ -291,10 +296,12 @@ impl NodeTSpan {
             return true;
         }
 
+        let params = draw_ctx.get_view_params();
+
         if values.text_gravity_is_vertical() {
-            *length += self.dy.get().normalize(values, draw_ctx);
+            *length += self.dy.get().normalize(values, &params);
         } else {
-            *length += self.dx.get().normalize(values, draw_ctx);
+            *length += self.dx.get().normalize(values, &params);
         }
 
         measure_children(node, cascaded, draw_ctx, length, usetextonly)
@@ -303,8 +310,8 @@ impl NodeTSpan {
     fn render(
         &self,
         node: &RsvgNode,
-        cascaded: &CascadedValues,
-        draw_ctx: &mut DrawingCtx,
+        cascaded: &CascadedValues<'_>,
+        draw_ctx: &mut DrawingCtx<'_>,
         x: &mut f64,
         y: &mut f64,
         usetextonly: bool,
@@ -312,8 +319,10 @@ impl NodeTSpan {
     ) -> Result<(), RenderingError> {
         let values = cascaded.get();
 
-        let mut dx = self.dx.get().normalize(values, draw_ctx);
-        let mut dy = self.dy.get().normalize(values, draw_ctx);
+        let params = draw_ctx.get_view_params();
+
+        let mut dx = self.dx.get().normalize(values, &params);
+        let mut dy = self.dy.get().normalize(values, &params);
 
         let vertical = values.text_gravity_is_vertical();
         let anchor = values.text_anchor;
@@ -321,7 +330,7 @@ impl NodeTSpan {
         let offset = anchor_offset(node, cascaded, draw_ctx, anchor, usetextonly);
 
         if let Some(self_x) = self.x.get() {
-            *x = self_x.normalize(values, draw_ctx);
+            *x = self_x.normalize(values, &params);
             if !vertical {
                 *x -= offset;
                 dx = match anchor {
@@ -334,7 +343,7 @@ impl NodeTSpan {
         *x += dx;
 
         if let Some(self_y) = self.y.get() {
-            *y = self_y.normalize(values, draw_ctx);
+            *y = self_y.normalize(values, &params);
             if vertical {
                 *y -= offset;
                 dy = match anchor {
@@ -351,7 +360,7 @@ impl NodeTSpan {
 }
 
 impl NodeTrait for NodeTSpan {
-    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
+    fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag<'_>) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
                 Attribute::X => self
@@ -476,7 +485,7 @@ impl From<WritingMode> for pango::Gravity {
 }
 
 fn create_pango_layout(
-    draw_ctx: &DrawingCtx,
+    draw_ctx: &DrawingCtx<'_>,
     values: &ComputedValues,
     text: &str,
 ) -> pango::Layout {
@@ -518,8 +527,10 @@ fn create_pango_layout(
 
     font_desc.set_stretch(pango::Stretch::from(values.font_stretch));
 
+    let params = draw_ctx.get_view_params();
+
     font_desc.set_size(to_pango_units(
-        values.font_size.0.normalize(values, draw_ctx),
+        values.font_size.0.normalize(values, &params),
     ));
 
     let layout = pango::Layout::new(&pango_context);
@@ -529,7 +540,7 @@ fn create_pango_layout(
 
     attr_list.insert(
         pango::Attribute::new_letter_spacing(to_pango_units(
-            values.letter_spacing.0.normalize(values, draw_ctx),
+            values.letter_spacing.0.normalize(values, &params),
         )).unwrap(),
     );
 
@@ -553,8 +564,8 @@ fn create_pango_layout(
 
 fn anchor_offset(
     node: &RsvgNode,
-    cascaded: &CascadedValues,
-    draw_ctx: &mut DrawingCtx,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx<'_>,
     anchor: TextAnchor,
     textonly: bool,
 ) -> f64 {
@@ -576,8 +587,8 @@ fn anchor_offset(
 
 fn measure_children(
     node: &RsvgNode,
-    cascaded: &CascadedValues,
-    draw_ctx: &mut DrawingCtx,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx<'_>,
     length: &mut f64,
     textonly: bool,
 ) -> bool {
@@ -601,8 +612,8 @@ fn measure_children(
 
 fn measure_child(
     node: &RsvgNode,
-    cascaded: &CascadedValues,
-    draw_ctx: &mut DrawingCtx,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx<'_>,
     length: &mut f64,
     textonly: bool,
 ) -> bool {
@@ -658,8 +669,8 @@ fn measure_child(
 
 fn render_children(
     node: &RsvgNode,
-    cascaded: &CascadedValues,
-    draw_ctx: &mut DrawingCtx,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx<'_>,
     x: &mut f64,
     y: &mut f64,
     textonly: bool,
@@ -678,8 +689,8 @@ fn render_children(
 
 fn render_child(
     node: &RsvgNode,
-    cascaded: &CascadedValues,
-    draw_ctx: &mut DrawingCtx,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx<'_>,
     x: &mut f64,
     y: &mut f64,
     textonly: bool,
@@ -745,6 +756,7 @@ pub extern "C" fn rsvg_node_chars_new(raw_parent: *const RsvgNode) -> *const Rsv
     boxed_node_new(
         NodeType::Chars,
         raw_parent,
+        "rsvg_chars",
         None,
         None,
         Box::new(NodeChars::new()),
